/************************************************************************
*
*                          MLXIOBSP.C
*
*			Example for usage of data acquisition library MLXIO.DLL.
*				This file is part of SORCUS application note 068
*                                                                      
*					      version      : see history                 
*							creation 	 : 29.09.99
*							last change  : see history
*							author       : M.Sailer
*
* 		(c) by SORCUS Computer GmbH Heidelberg 1999
*************************************************************************/

/************************************************************************
*
*	software requirements:
*		- SORCUS I/O port drivers for Multi-LAB/2 or Modular-4/486
*		- library MLXW32.DLL
*		- library MLXIO.DLL
*		- additional OSX programs as listed in SORCUS application note 068
*		- Microsoft Visual C++, >= Version 4.2 
*		- import libraries MLXIO.LIB and MLXW32.LIB
*
*************************************************************************/

/************************************************************************
*  history:
*   	1.A.000 29.09.99
*        - creation of the file
*
*************************************************************************/
// include standard C libraries
#include <stdio.h>
#include <math.h>
#include <conio.h>
// include SORCUS libraries
#include "mlxw32.h"
#include "mlxio.h"


/*************************************************************************
*           type definitions
*************************************************************************/

// we define a description table for input channels:
typedef struct _OUTPUTCHANNEL
{
	HMLXIO hOut;					// Handle of output channel
	ULONG  ulNumOfSamples;		// number of samples sent
	DOUBLE dFrequencyFactor;	// used to calculate sine wave
	ULONG	 ulChannelID;			// user-defined number of channel
}
OUTPUTCHANNEL, *POUTPUTCHANNEL;

// we define a description table for input channels:
typedef struct _INPUTCHANNEL
{
	HMLXIO hIn;						// Handle of input channel
	DOUBLE dFactor;				// used to calculate input voltage 
										// from 12-bit samples
	ULONG  ulNumOfSamples;		// number of samples received
	ULONG	 ulChannelID;			// user-defined number of channel
}
INPUTCHANNEL, *PINPUTCHANNEL;

#define SLOTNO		2

/*************************************************************************
*	ErrorHandler
*
*	PURPOSE
*	display errors received from SORCUS library MLXW32.DLL
*
*************************************************************************/
void  	ErrorHandler (void)
{          
	char   szString [256];
	
	struct ml8_error_info_type  err;
	
	ml8_get_error_info( &err);
	                  
	switch (err.errortype) {
	case ERROR_DLL:
		wsprintf (szString, "DLL-Error:\nclass: %02Xh\ncode: %02Xh",
			err.dll.errorclass, err.dll.errorcode);
		break;
	case ERROR_COMMUNICATION:
		wsprintf (szString, "Communication Error:\nclass: %02Xh\ncode: %02Xh",
			err.dll.errorclass, err.dll.errorcode);
		break;
	case ERROR_SYSTEM:
		wsprintf (szString, "System Error: %d(d)\n", err.system.error);
		break;
	case ERROR_DRIVER:
		wsprintf (szString, "Driver Error: \n"
			"expected data length: %d(d)\n" 
			"real data length: %d(d)", 
			err.drv.dataoutsize, err.drv.retlength);
		break;
	default: wsprintf(szString, "unknown error");
	}
	
	MessageBox (GetActiveWindow (), szString, "ML8 error message", MB_OK | MB_ICONSTOP);

	ml8_clear_error();
}

/*************************************************************************
*	CallbackOutput
*
*	PURPOSE
*	this routine is called when an output channel needs to be filled.
*  In this example, the sample size of the channel is assumed to be 2
*  (= sizeof (SHORT))
*
*************************************************************************/

void	MLXIO_CALLBACK	CallbackOutput	(HMLXIO hChan)
{
	POUTPUTCHANNEL pOut;
	SHORT	buffer [256];
	ULONG	error, free, i;

	// get pointer of the local output channel description table
	pOut = (POUTPUTCHANNEL)mlxio_get_private_data (hChan);

	do
	{
		// check if channel needs to be filled
		mlxio_get_channel_state (hChan, &free, NULL, NULL);

		free /= sizeof (SHORT);

		if (free)
		{
			if (free > 256)
			{
				free = 256;
			}

			// generate sine data buffer
			for (i=0;i<free;i++)
			{
				buffer [i] = (SHORT)
					(2047.0 * 
						sin (pOut->dFrequencyFactor * (DOUBLE)(pOut->ulNumOfSamples + i)));
			}

			// write it into PC buffer
			i = mlxio_write_data (hChan, buffer, free * sizeof (SHORT), &error);

			if (error)
			{
				// something went wrong
				printf ("Error %X in channel %d\n", error, pOut->ulChannelID);
				mlxio_reset_error (hChan);
			}

			// i: number of bytes written

			i /= sizeof (USHORT);

			// i is now number of samples written

			pOut->ulNumOfSamples += i;
				
			printf ("Channel %d: %d samples written\n", pOut->ulChannelID, i);
		}
	}
	while (free); // as long as PC buffer can receive data
}

/*************************************************************************
*	CallbackInput
*
*	PURPOSE
*	this routine is called when an input channel has received data from
*  SORCUS board.
*
*************************************************************************/

void	MLXIO_CALLBACK	CallbackInput	(HMLXIO hChan)
{
	SHORT buffer [256];
	ULONG numofbytes, error;
	PINPUTCHANNEL pIn;
	LONG	sum;
	ULONG count, index, samples_in_buffer;
	DOUBLE average;

	// get pointer of the local input channel description table
	pIn = (PINPUTCHANNEL)mlxio_get_private_data (hChan);

	do
	{
		// read data from PC buffer
		numofbytes = mlxio_read_data
			((HMLXIO)hChan, buffer, sizeof (buffer), &error, NULL);

		if (error)
		{
			// something went wrong
			printf ("Error %X in channel %d\n", error, pIn->ulChannelID);
			mlxio_reset_error (hChan);
		}

		if (numofbytes)
		{
			samples_in_buffer = numofbytes / sizeof(SHORT);

			pIn->ulNumOfSamples += samples_in_buffer;

			sum = 0;
			count = 0;
				
			for (index=0; index<samples_in_buffer; index++)
			{
				sum += buffer [index];
				count++;
			}

			if (count)
			{
				average = (DOUBLE)sum / (DOUBLE)count * pIn->dFactor;

				printf ("Channel %d: received %d samples with average %.2f V\n",
					pIn->ulChannelID, count, average);
			}
		}
	}
	while (numofbytes);
}

/*************************************************************************
*	main
*
*	PURPOSE
*	- initialize libraries
*	- open and prepare channels
*	- start measurement
*	- wait for user input
*	- stop measurement
*	- close library MLXIO.DLL
*
*************************************************************************/

void	main	(void)
{
	USHORT result;
	ULONG  error;
	INPUTCHANNEL In1, In2;
	OUTPUTCHANNEL Out1;
	CPS_M5B1U cps;

	// initialize library MLXW32.DLL
	ml8_bib_startup (LANGUAGE_C);

	// reset board 0 and start communication
	result = ml8_reset (0, // board number
							  0, // I/O address, not used in WIN32
							  1, // mode: use interrupt
							  0, // irq channel, not used in WIN32
							  0, // dma channel, not used in WIN32
							  ErrorHandler, // error callback procedure
							  "ROM", // load board operating system from ROM
							  30); // timeout value

	if (result != 0)
	{
		printf ("Fehler %04X bei ml8_reset\n");
		exit (0);
	}

	// the following call of mlxio_set_driver_path is only necessary if
	// you don't have installed SNW32.EXE and your additional OSX 
	// programs (drivers for MLXIO.DLL and MDDs) are not placed in
	// the directory where MLXIO.DLL is loaded from:
	
	// mlxio_set_driver_path	("C:\\SORCUS\\MDD");

	// initialize MLXIO.DLL:
	// base sample frequency is 2 kHz,
	// PC signal buffers can hold data 4000/2000 = 2s long

	error = mlxio_initialize (2000.0, 4000, MLXIO_CONTINUOUS | MLXIO_INPUT_FLEX);
	
	if (error != MLXIO_OK)
	{
		printf ("Error during mlxio_initialize: %d\n", error);
		exit (0);
	}

	// define first channel to use:
	// single ended input AIN-0

	cps.usDevice = DEVICE_AIN_SE; // single ended input
	cps.usFlags = 0;
   cps.usIndexFirst = 0;			// we use AIN-0 of
	cps.usIndexLast  = 0;			// module M-5B-1/U 
   cps.usReadMode   = IO_MODE_DIRECT;
   cps.usWriteMode  = 0;
   cps.usSettleTime = 5;			// settle time = 5 * 5 us
   cps.usRange = RANGE_BIP_5V;	// range -5 ... +5 V
					  
	In1.hIn = 
		mlxio_open_input_channel 
			(SLOTNO,		// module M-5B-1/U is placed on slot 'SLOTNO' of Modular-4/486
			 sizeof (cps), 
			 &cps, 
			 1);			// sample rate of this channel is 2000.0 / 1 Hz

	if (!In1.hIn)
	{
		printf ("Unable to open input channel AIN-0 of module M-5B-1/U on slot %d\n", SLOTNO);
		exit (0);
	}

	// now set channel callback routine
	mlxio_set_chan_callback (In1.hIn, CallbackInput);

	// save pointer to struct INPUTCHANNEL as private data
	mlxio_set_private_data (In1.hIn, (ULONG)&In1);

	In1.dFactor = 10.0 / 4095.0;
	In1.ulNumOfSamples = 0;
	In1.ulChannelID = 0;
	  
	// define second channel to use:
	// AIN-2 as single ended input
	
	cps.usDevice = DEVICE_AIN_SE;	// single ended input
	cps.usFlags = 0;
   cps.usIndexFirst = 2;			// AIN-2
	cps.usIndexLast  = 2;			
   cps.usReadMode   = IO_MODE_DIRECT;
   cps.usWriteMode  = 0;
   cps.usSettleTime = 5;			// settle time = 5 * 5 us
   cps.usRange = RANGE_BIP_5V;	// range -5 ... +5 V

	In2.hIn = 
		mlxio_open_input_channel 
			(SLOTNO,			// module M-5B-1/U is placed on slot 'SLOTNO' of Modular-4/486
			 sizeof (cps), 
			 &cps, 
			 2);			// sample rate of this channel is 2000.0 / 2 -> 1000 Hz

	if (!In2.hIn)
	{
		printf ("Unable to open AIN-2 of module M-5B-1/U on slot %d\n", SLOTNO);
		exit (0);
	}

	// now set channel callback routine
	mlxio_set_chan_callback (In2.hIn, CallbackInput);

	// save pointer to struct INPUTCHANNEL as private data
	mlxio_set_private_data (In2.hIn, (ULONG)&In2);

	In2.dFactor = 10.0 / 4095.0;
	In2.ulNumOfSamples = 0;
	In2.ulChannelID = 1;
	  
	// define third channel to use:
	// analog output channel AOUT-0
	
	cps.usDevice = DEVICE_AOUT;	// analog output channel
	cps.usFlags = 0;
   cps.usIndexFirst = 0;
	cps.usIndexLast  = 0;
   cps.usReadMode   = IO_MODE_DIRECT;
   cps.usWriteMode  = IO_MODE_DIRECT;
   cps.usSettleTime = 0;
   cps.usRange = RANGE_BIP_5V;	// maybe you have to change this to
											// RANGE_UPP_5V or RANGE_UPP_10V 
											// according to your jumper/eeprom settings

/*	cps.usDevice = DEVICE_AOUT;
	cps.usFlags = _CP_EXCLUSIVE;
	cps.usIndexFirst = 4;
	cps.usIndexLast = 4;
	cps.usWriteMode = IO_MODE_DIRECT;*/

	Out1.hOut = 
		mlxio_open_output_channel 
			(SLOTNO,			// module M-5B-1/U is placed on slot 'SLOTNO' of Modular-4/486
			 sizeof (cps), 
			 &cps, 
			 10);			// sample rate of this channel is 2000.0 / 10 -> 200 Hz

	if (!Out1.hOut)
	{
		printf ("Unable to open channel AOUT-0 of module M-5B-1/U on slot %d\n", SLOTNO);
		exit (0);
	}

	// now set channel callback routine
	mlxio_set_chan_callback (Out1.hOut, CallbackOutput);

	// save pointer to struct INPUTCHANNEL as private data
	mlxio_set_private_data (Out1.hOut, (ULONG)&Out1);

	Out1.ulNumOfSamples = 0;
	
	// we want to generate a sine wave with 0.1 Hz, based on
	// channel sample frequency of 200 Hz
	Out1.dFrequencyFactor = 2.0 * 3.141592 / 200.0 * 0.1;

	Out1.ulChannelID = 2;
	
	// before starting measurement, we call the 
	// output channel callback routine directly to prefill the channel:

	CallbackOutput (Out1.hOut);

	error = mlxio_start_measurement ();
	
	if (error != MLXIO_OK)
	{
		printf ("Error during mlxio_start_measurement: %d\n", error);
		exit (0);
	}

	// measurement is running: 
	// wait until user presses ESC
	while (getch() != 27);

	mlxio_stop_measurement();

	mlxio_close_all ();
}

// end of file MLXIOBSP.C